  module rand_mod
  implicit none
  integer, parameter :: gl = 80    !!!  set the parameter for minimum GL for 50-60 GeV photons.
  integer, parameter :: north_south = 1     !!! 1 for north, -1 for south
  character, parameter :: in_rootname*20 = '../data_set/jan_LAT_'   
  character, parameter :: out_rootname*21 = '../data_res/rand_jan_'   
  character, parameter :: i_n1_filename*80='10GeV_clean_deg.dat', i_n2_filename*80='20GeV_clean_deg.dat'
  character, parameter :: i_n3_filename*80='30GeV_clean_deg.dat', i_n4_filename*80='40GeV_clean_deg.dat'
  character, parameter :: i_n5_filename*80='50GeV_clean_deg.dat'
  integer, parameter :: sample_num=10000
  integer, parameter :: max_Rw = 20
  character, parameter :: point_n_filename*80 = 'N_source.dat'   !!! source position in the north
  character, parameter :: point_s_filename*80 = 'S_source.dat'   !!! source position in the south

  real(8),parameter :: const_pi = 3.14159265359  
  real(8),parameter :: glat_max=90.d0/180.d0*const_pi, glat_min=DBLE(gl)/180.d0*const_pi

  character :: output_filenames(6)*80
  integer :: fix_num 
  real(8), parameter :: R_fix = 1.5d0
  real(8), dimension(:,:), allocatable :: fix_list
  contains

  subroutine rand_samp
  implicit none
  integer :: data_n1n, data_n1s, data_n2n, data_n2s, data_n3n, data_n3s
  integer :: data_n4n, data_n4s, data_n5n, data_n5s
  real(8), dimension(:,:), allocatable :: n1n_list, n1s_list
  real(8), dimension(:,:), allocatable :: n2n_list, n2s_list
  real(8), dimension(:,:), allocatable :: n3n_list, n3s_list
  real(8), dimension(:,:), allocatable :: n4n_list, n4s_list
  real(8), dimension(:,:), allocatable :: n5n_list, n5s_list
  character :: n1_filename*80, n2_filename*80, n3_filename*80, n4_filename*80, n5_filename*80 
  real(8), dimension(:,:), allocatable :: n1_list, n2_list, n3_list, n4_list, n5_list, ntotal_list
  integer :: data_n1, data_n2, data_n3, data_n4, data_n5, data_total
  integer :: i, j, k, l

  real(8) :: dsum(6), sums(6), final_ans(max_RW,6)
  real(8) :: Rc, Rw
  real(8), dimension(5) :: n1, n2, n3, n4, n5
  real(8), dimension(6, 5) :: nn
  real(8), dimension(:), allocatable :: rand
  real(8), dimension(sample_num)  :: final_re
  integer :: cn1, cn2, cn3, cn4, cn5

  Rw = DBLE(max_Rw)/180.d0*const_pi

  n1_filename = in_rootname//i_n1_filename
  n2_filename = in_rootname//i_n2_filename
  n3_filename = in_rootname//i_n3_filename
  n4_filename = in_rootname//i_n4_filename
  n5_filename = in_rootname//i_n5_filename
  call read_data_num_NS(data_n1n, data_n1s, n1_filename)
  call read_data_num_NS(data_n2n, data_n2s, n2_filename)
  call read_data_num_NS(data_n3n, data_n3s, n3_filename)
  call read_data_num_NS(data_n4n, data_n4s, n4_filename)
  call read_data_num_NS(data_n5n, data_n5s, n5_filename)
  allocate(n1n_list(2,data_n1n))
  allocate(n1s_list(2,data_n1s))
  allocate(n2n_list(2,data_n2n))
  allocate(n2s_list(2,data_n2s))
  allocate(n3n_list(2,data_n3n))
  allocate(n3s_list(2,data_n3s))
  allocate(n4n_list(2,data_n4n))
  allocate(n4s_list(2,data_n4s))
  allocate(n5n_list(2,data_n5n))
  allocate(n5s_list(2,data_n5s))
  call read_data_NS(n1n_list, n1s_list, n1_filename)
  call read_data_NS(n2n_list, n2s_list, n2_filename)
  call read_data_NS(n3n_list, n3s_list, n3_filename)
  call read_data_NS(n4n_list, n4s_list, n4_filename)
  call read_data_NS(n5n_list, n5s_list, n5_filename)

  if (north_south > 0.) then
  call read_list_num(data_n1, n1n_list, RW)
  call read_list_num(data_n2, n2n_list, RW)
  call read_list_num(data_n3, n3n_list, RW)
  call read_list_num(data_n4, n4n_list, RW)
  call read_list_num(data_n5, n5n_list, 0.d0)
  allocate(n1_list(5,data_n1))
  allocate(n2_list(5,data_n2))
  allocate(n3_list(5,data_n3))
  allocate(n4_list(5,data_n4))
  allocate(n5_list(5,data_n5))

  call read_data_hole_num(fix_num, point_n_filename, Rw)
  allocate(fix_list(5, fix_num))
  call read_data_hole(fix_list, point_n_filename, Rw)

  write (output_filenames(1), '(i2.2,"_125_n.dat")') gl
  write (output_filenames(2), '(i2.2,"_135_n.dat")') gl
  write (output_filenames(3), '(i2.2,"_145_n.dat")') gl
  write (output_filenames(4), '(i2.2,"_235_n.dat")') gl
  write (output_filenames(5), '(i2.2,"_245_n.dat")') gl
  write (output_filenames(6), '(i2.2,"_345_n.dat")') gl

  else
  call read_list_num(data_n1, n1s_list, RW)
  call read_list_num(data_n2, n2s_list, RW)
  call read_list_num(data_n3, n3s_list, RW)
  call read_list_num(data_n4, n4s_list, RW)
  call read_list_num(data_n5, n5s_list, 0.d0)
  allocate(n1_list(5,data_n1))
  allocate(n2_list(5,data_n2))
  allocate(n3_list(5,data_n3))
  allocate(n4_list(5,data_n4))
  allocate(n5_list(5,data_n5))

  call read_data_hole_num(fix_num, point_s_filename, Rw)
  allocate(fix_list(5, fix_num))
  call read_data_hole(fix_list, point_s_filename, Rw)

  write (output_filenames(1), '(i2.2,"_125_s.dat")') gl
  write (output_filenames(2), '(i2.2,"_135_s.dat")') gl
  write (output_filenames(3), '(i2.2,"_145_s.dat")') gl
  write (output_filenames(4), '(i2.2,"_235_s.dat")') gl
  write (output_filenames(5), '(i2.2,"_245_s.dat")') gl
  write (output_filenames(6), '(i2.2,"_345_s.dat")') gl

  end if

  data_total= data_n1 + data_n2 + data_n3 + data_n4
  allocate(ntotal_list(5,data_total))

  output_filenames(1) = out_rootname//output_filenames(1) 
  output_filenames(2) = out_rootname//output_filenames(2) 
  output_filenames(3) = out_rootname//output_filenames(3) 
  output_filenames(4) = out_rootname//output_filenames(4) 
  output_filenames(5) = out_rootname//output_filenames(5) 
  output_filenames(6) = out_rootname//output_filenames(6) 

  open(unit=1000,file=output_filenames(1),status='replace')
  open(unit=2000,file=output_filenames(2),status='replace')
  open(unit=3000,file=output_filenames(3),status='replace')
  open(unit=4000,file=output_filenames(4),status='replace')
  open(unit=5000,file=output_filenames(5),status='replace')
  open(unit=6000,file=output_filenames(6),status='replace')


  do l=1, sample_num
  sums(:) = 0.d0
  Rw = DBLE(max_Rw)/180.d0*const_pi
  call make_randum_dat_fix(n5_list, data_n5,1,0.d0)
  call make_randum_dat_fix(ntotal_list, data_total,3, Rw)

  do i=1, data_total
  If (i < data_n1+1) then 
  n1_list(:,i)=ntotal_list(:,i)
  elseif (i < data_n1 + data_n2 +1) then
  j = i -data_n1
  n2_list(:,j)=ntotal_list(:,i)
  elseif (i < data_n1 + data_n2 + data_n3 +1) then
  j = i - data_n1 - data_n2
  n3_list(:,j)=ntotal_list(:,i)
  else 
  j = i - data_n1 - data_n2 - data_n3
  n4_list(:,j)=ntotal_list(:,i) 
  end if
  end do
 
  do j=1, max_Rw
  Rw = DBLE(j)/180.d0*const_pi
  do i=1, data_n5
  
  n5(1)=n5_list(1,i)
  n5(2)=n5_list(2,i)
  n5(3)=n5_list(3,i)
  n5(4)=n5_list(4,i)
  n5(5)=n5_list(5,i)

  call sum_vec(n1, n5, Rw, n1_list, cn1)
  call sum_vec(n2, n5, Rw, n2_list, cn2)
  call sum_vec(n3, n5, Rw, n3_list, cn3)
  call sum_vec(n4, n5, Rw, n4_list, cn4)
 
  nn(1,1) = n1(2)*n2(3) - n1(3)*n2(2)
  nn(1,2) = n1(3)*n2(1) - n1(1)*n2(3)  
  nn(1,3) = n1(1)*n2(2) - n1(2)*n2(1)

  nn(2,1) = n1(2)*n3(3) - n1(3)*n3(2)
  nn(2,2) = n1(3)*n3(1) - n1(1)*n3(3)  
  nn(2,3) = n1(1)*n3(2) - n1(2)*n3(1)

  nn(3,1) = n1(2)*n4(3) - n1(3)*n4(2)
  nn(3,2) = n1(3)*n4(1) - n1(1)*n4(3)  
  nn(3,3) = n1(1)*n4(2) - n1(2)*n4(1)

  nn(4,1) = n2(2)*n3(3) - n2(3)*n3(2)
  nn(4,2) = n2(3)*n3(1) - n2(1)*n3(3)  
  nn(4,3) = n2(1)*n3(2) - n2(2)*n3(1)

  nn(5,1) = n2(2)*n4(3) - n2(3)*n4(2)
  nn(5,2) = n2(3)*n4(1) - n2(1)*n4(3)  
  nn(5,3) = n2(1)*n4(2) - n2(2)*n4(1)

  nn(6,1) = n3(2)*n4(3) - n3(3)*n4(2)
  nn(6,2) = n3(3)*n4(1) - n3(1)*n4(3)  
  nn(6,3) = n3(1)*n4(2) - n3(2)*n4(1)
  
  do k=1,6
  dsum(k) = n5(1)*nn(k,1) + n5(2)*nn(k,2) + n5(3)*nn(k,3)
  sums(k) = sums(k) + dsum(k)
  end do

  end do  ! for i (data_n5)

  final_ans(j,:) = sums(:)/DBLE(data_n5)
  sums(:) = 0.d0
  end do  ! for j (Rw)

  write(1000,'(20E15.5, i8)') final_ans(1:max_Rw,1), data_n5
  write(2000,'(20E15.5, i8)') final_ans(1:max_Rw,2), data_n5
  write(3000,'(20E15.5, i8)') final_ans(1:max_Rw,3), data_n5
  write(4000,'(20E15.5, i8)') final_ans(1:max_Rw,4), data_n5
  write(5000,'(20E15.5, i8)') final_ans(1:max_Rw,5), data_n5
  write(6000,'(20E15.5, i8)') final_ans(1:max_Rw,6), data_n5

  end do   ! for l

  end subroutine
  
  subroutine sum_vec(nn, n1, Rw, nn_list, cn)
  implicit none
  real(8) :: nn_list(:,:)
  real(8) :: Rw, Rc
  real(8), dimension(5) :: n1, n2, nn
  integer i, data_num, cn
  
  data_num = size(nn_list(1,:))

  cn=0
  nn = 0.d0
  do i = 1, data_num
  n2(1)=nn_list(1,i)
  n2(2)=nn_list(2,i)
  n2(3)=nn_list(3,i)
  n2(4)=nn_list(4,i)
  n2(5)=nn_list(5,i)

  call window_R(n1,n2,Rw,Rc)

  if (Rc > 1.d0) then
  nn(1) = nn(1) + n2(1)
  nn(2) = nn(2) + n2(2)
  nn(3) = nn(3) + n2(3)
  cn = cn +1
  end if
  end do

  IF (cn > 0) then
  nn(1) = nn(1)/DBLE(cn)
  nn(2) = nn(2)/DBLE(cn)
  nn(3) = nn(3)/DBLE(cn)
  else
  nn = 0.d0
  end if

  end subroutine

  subroutine read_data(nn_list, filename,Rw)
  implicit none
  integer i,io, data_num
  real(8) :: nn_list(:,:)
  real(8) :: theta, phi, Rw
  character :: filename*80

  data_num = size(nn_list(1,:))
  open(1000,file=filename,status='old')
  i=0
  do 
  read(1000,*,IOSTAT=io) phi, theta
  phi = abs(phi)/180.*const_pi
  theta = abs(theta)/180.*const_pi

  if (theta > glat_min - Rw) then
  i=i+1
  nn_list(1,i) = sin(const_pi/2.-theta)*cos(phi)
  nn_list(2,i) = sin(const_pi/2.-theta)*sin(phi)
  nn_list(3,i) = cos(const_pi/2.-theta)
  nn_list(4,i) = theta
  nn_list(5,i) = phi
  if (i == data_num) exit
  end if
  end do
  print*, "confirmation: ",data_num, " in ", filename
  close(1000)
  end subroutine

  subroutine read_data_num(data_num, filename, Rw)
  implicit none
  integer i,io, data_num, j
  real(8) :: phi, theta, z, Rw
  character :: filename*80
  
  open(1000,file=filename,status='old')
  i=0
  j=0
  do 
  read(1000,*,IOSTAT=io) phi, theta
  phi = abs(phi)/180.*const_pi
  theta = abs(theta)/180.*const_pi
  if (io .lt. 0) exit
  if (abs(theta) > glat_min - Rw)i=i+1
  j=j+1
  enddo
  data_num = i
  print*, data_num, "/", j,  " in ", filename
  close(1000)
  end subroutine

  subroutine read_data_hole_num(data_num, filename, Rw)
  implicit none
  integer i,io, data_num, j
  real(8) :: phi, theta, z, Rw
  character :: filename*80
  
  open(1000,file=filename,status='old')
  i=0
  j=0
  do 
  read(1000,*,IOSTAT=io) phi, theta
  phi=phi/180.d0*const_pi
  theta=abs(theta)/180.d0*const_pi
  if (io .lt. 0) exit
  if (theta > (glat_min- Rw - R_fix/180.d0*const_pi))i=i+1
  j=j+1
  enddo
  data_num = i
  print*, data_num, "/", j,  " in ", filename
  close(1000)
  end subroutine

  subroutine read_data_hole(nn_list, filename, Rw)
  implicit none
  integer i,io, data_num
  real(8) :: nn_list(:,:)
  real(8) :: theta, phi, Rw
  character :: filename*80

  data_num = size(nn_list(1,:))
  open(1000,file=filename,status='old')
  i=0
  do 
  read(1000,*,IOSTAT=io) phi, theta
  phi=phi/180.d0*const_pi
  theta=abs(theta)/180.d0*const_pi

  if (theta > glat_min- Rw -R_fix/180.d0*const_pi ) then
  i=i+1
  nn_list(1,i) = sin(const_pi/2.-theta)*cos(phi)
  nn_list(2,i) = sin(const_pi/2.-theta)*sin(phi)
  nn_list(3,i) = cos(const_pi/2.-theta)
  nn_list(4,i) = theta
  nn_list(5,i) = phi
  if (i == data_num) exit
  end if
  end do
  print*, "confirmation: ",data_num, " in ", filename
  close(1000)
  end subroutine


  subroutine window_R(n1, n2, Rw, Rc)
  implicit none
  real(8) :: x, y, z, d2, Rc, Rw
  real(8), dimension(5) :: n1, n2

  d2 = n1(1)*n2(1) + n1(2)*n2(2) + n1(3)*n2(3)

  if (abs(d2) > cos(Rw)) then
  Rc = 10.d0
  else 
  Rc = 0.d0
  end if
  end subroutine

  Subroutine randum_num(rand, in)
   integer :: clock 
   integer :: nRand 
   integer, allocatable :: seed(:) 
   real(kind = 8) :: rand(:) 
   integer :: i, in

   call random_seed(size = nRand)
   allocate(seed(nRand))
   call system_clock(count = clock)
   seed = clock + in
   call random_seed(put = seed)
   call random_number(rand) 
   deallocate(seed)
  end subroutine

  subroutine make_randum_dat_fix(nn_list, data_nn, in, Rw)
  implicit none
  real(8), dimension(:,:) :: nn_list
  integer :: data_nn, i, in, j, k
  real(8), dimension(:), allocatable ::rand
  real(8) :: phi, theta, norm, amax, amin, patch_size, Rc, Rw
  logical :: check_t
  real(8), dimension(5) ::nn, nf
  
  allocate(rand(1000*data_nn))
  call randum_num(rand, in)
   patch_size=R_fix/180.d0*const_pi

  amax=sin(glat_max)
  amin=sin(glat_min-Rw)
  norm = amax-amin
  j=0
  do i = 1, data_nn
   do   
    j = j+1
    theta = asin(rand(j)*norm+amin)
    j = j+1
    phi = rand(j)*2.d0*const_pi
    nn(1) = sin(const_pi/2.-theta)*cos(phi)
    nn(2) = sin(const_pi/2.-theta)*sin(phi)
    nn(3) = cos(const_pi/2.-theta)
    nn(4) = theta
    nn(5) = phi

    check_t = .False.
	do k=1, fix_num
	Rc=0.
    nf(1)=fix_list(1,k)
    nf(2)=fix_list(2,k)
    nf(3)=fix_list(3,k)
    nf(4)=fix_list(4,k)
    nf(5)=fix_list(5,k)

    call window_R(nn, nf, patch_size, Rc)
    if (RC > 1.) check_t = .False.
    if (RC > 1.) exit
    check_t = .True.
    end do
  if (check_t .eqv. .True.) exit
   end do

  nn_list(1,i) = nn(1)
  nn_list(2,i) = nn(2)
  nn_list(3,i) = nn(3)
  nn_list(4,i) = nn(4)
  nn_list(5,i) = nn(5)
  
  end do
  deallocate(rand)
  end subroutine

  subroutine read_list_num(data_num, data_list, Rw)
  implicit none
  integer i, j, k, data_num
  real(8) :: phi, theta, z, Rw
  real(8), dimension(:,:) :: data_list
  
  i=0
  do  j=1, size(data_list(1,:))
  phi = data_list(1,j) /180.d0*const_pi
  theta = data_list(2,j) /180.d0*const_pi
  if (abs(theta) > abs(glat_min - Rw)) i=i+1
  enddo
  data_num = i

  end subroutine


  subroutine read_data_NS(nnN_list, nnS_list, filename)
  implicit none
  integer i, j, k, io, datan_num, datas_num, datat_num
  real(8), dimension(:,:) :: nnN_list, nnS_list
  real(8) :: theta, phi
  character :: filename*80

  datan_num = size(nnN_list(1,:))
  datas_num = size(nnS_list(1,:))
  datat_num = datan_num + datas_num

  open(1000,file=filename,status='old')
  i=0
  j=0
  k=0
  do 
  read(1000,*,IOSTAT=io) phi, theta
  k=k+1
  if (theta > 0.) then
  i=i+1    
  nnN_list(1,i) = phi
  nnN_list(2,i) = theta
  else
  j=j+1
  nnS_list(1,j) = phi
  nnS_list(2,j) = theta
  end if

  if (k == datat_num) exit
  end do
  close(1000)
  end subroutine


  subroutine read_data_num_NS(datan_num, datas_num, filename)
  implicit none
  integer i, j, k, io, datan_num, datas_num
  real(8) :: phi, theta
  character :: filename*80
  
  open(1000,file=filename,status='old')
  i=0
  j=0
  k=0
  do 
  read(1000,*,IOSTAT=io) phi, theta  
  if (io .lt. 0) exit
  if (theta >0. ) then
  i=i+1
  else
  j=j+1
  end if
  k=k+1
  enddo
  datan_num = i
  datas_num = j
  close(1000)

  end subroutine



end module
